package com.yskj.acdr.master.chat.controller;

import cn.dev33.satoken.stp.StpUtil;
import cn.dev33.satoken.util.SaFoxUtil;
import com.yskj.acdr.common.cache.GlobalRedisCache;
import com.yskj.acdr.config.MyEndpointConfigurator;
import com.yskj.acdr.master.chat.chatenum.ChatState;
import com.yskj.acdr.master.chat.entity.ChatMessage;
import com.yskj.acdr.master.chat.entity.WSRes;
import com.yskj.acdr.master.chat.service.ChatMessageService;
import com.yskj.acdr.master.user.entity.Users;
import com.yskj.acdr.master.user.service.UsersService;
import jakarta.annotation.Resource;
import jakarta.websocket.*;
import jakarta.websocket.server.PathParam;
import jakarta.websocket.server.ServerEndpoint;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 客服系统或者是宠托师聊天功能实现
 */
@ServerEndpoint(value = "/chat/{token}", configurator = MyEndpointConfigurator.class)
@Component
@Slf4j
public class ChatServerEndpoint {

    private Session session;

    @Resource
    private UsersService usersService;

    @Resource
    private GlobalRedisCache<String> redisCache;

    @Resource
    private ChatMessageService cms;

    @Resource
    private PlatformTransactionManager transactionManager;  // 注入 PlatformTransactionManager

    private static final Map<Long, Session> sessionMap = new ConcurrentHashMap<>();

    private Long userId;

    /**
     * 收到客户端消息后调用的方法
     */
    @OnMessage
    public void onMessage(String message, Session session) throws IOException {
        try {
            // 处理消息并对服务进行消息转发
            Map<String, List<String>> parameter = session.getRequestParameterMap();
            List<String> userIds = parameter.get("toUserId");
            if (!userIds.isEmpty()) {
                // 获取转发消息的toUserId
                String toUserId = userIds.getFirst();
                Session toSession = sessionMap.get(Long.parseLong(toUserId));

                // 创建 TransactionDefinition 定义事务的属性
                DefaultTransactionDefinition def = new DefaultTransactionDefinition();
                def.setPropagationBehavior(TransactionDefinition.PROPAGATION_REQUIRED);

                // 开启事务
                TransactionStatus status = transactionManager.getTransaction(def);

                try {
                    ChatMessage chatMessage = new ChatMessage()
                            .setUserId(userId)
                            .setBeUserId(Long.parseLong(toUserId))
                            .setState(ChatState.NO_READ)
                            .setContent(message);

                    // 保存消息到数据库
                    boolean save = cms.save(chatMessage);

                    if (save) {
                        // 提交事务
                        transactionManager.commit(status);

                        // 如果对方在线，则发送消息
                        if (toSession != null) {
                            sendText(toSession, WSRes.success(chatMessage));
                        }
                    } else {
                        // 如果保存失败，回滚事务
                        transactionManager.rollback(status);
                        sendText(session, WSRes.error("发送消息失败，请检查网络连接！"));
                    }
                } catch (Exception e) {
                    // 在发生任何异常时回滚事务
                    transactionManager.rollback(status);
                    log.error(e.getMessage());
                    sendText(session, WSRes.error("发送消息失败，请检查网络连接！"));
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage());
            sendText(session, WSRes.error("发送消息失败，请检查网络连接！"));
        }
    }

    /**
     * 连接建立成功时调用的方法
     */
    @OnOpen
    public void onOpen(Session session, @PathParam("token") String token) throws IOException {
        this.session = session;

        // 使用 Sa-Token 进行登录检查
        try {
            Object loginIdByToken = StpUtil.getLoginIdByToken(token);
            if (loginIdByToken == null) {
                sendText(session, WSRes.error("未登录或登录已过期，连接将关闭"));
                session.close();
                return;
            }
            userId = SaFoxUtil.getValueByType(loginIdByToken, long.class);
            // 如果登录有效
            Users users = usersService.getById(userId);
            sendText(session, WSRes.success("尊敬的%s您好！连接成功，欢迎进入客服系统！".formatted(users.getNickname())));
            sessionMap.put(userId, session);
        } catch (Exception e) {
            sendText(session, WSRes.error("未登录或登录已过期，连接将关闭"));
            session.close();
        }
    }

    // 发送信息方法
    private void sendText(Session session, String text) {
        try {
            session.getBasicRemote().sendText(text);
        } catch (IOException e) {
            log.error(e.getMessage());
        }
    }

    /**
     * 连接关闭时调用的方法
     */
    @OnClose
    public void onClose() {
        if (userId != null) {
            sessionMap.remove(userId);
        }
    }

    /**
     * 发生错误时调用的方法
     */
    @OnError
    public void onError(Session session, Throwable error) {
        error.printStackTrace();
    }
}
