package com.yskj.acdr.master.file.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.yskj.acdr.common.cache.GlobalRedisCache;
import com.yskj.acdr.common.exception.BusinessException;
import com.yskj.acdr.master.file.entity.FileMap;
import com.yskj.acdr.master.file.mapper.FileMapMapper;
import com.yskj.acdr.master.file.service.FileMapService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yskj.acdr.utils.FileUtils;
import jakarta.annotation.Resource;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author 林河
 * @since 2024-08-14
 */
@Service
public class FileMapServiceImpl extends ServiceImpl<FileMapMapper, FileMap> implements FileMapService {

    @Resource
    private GlobalRedisCache<String> redisCache;

    // 从配置文件中获取自定义路径
    @Value("${path.file}")
    private String filePath;

    @Value("${admin.password}")
    private String adminPassword;

    @Value("${admin.key}")
    private String adminKey;

    @Value("${admin.url}")
    private String adminUrl;

    /**
     * 保存到本地服务器上面
     *
     * @param file 删除的文件
     * @return 上传结果
     */
    @Override
    public FileMap saveLocalFile(MultipartFile file) {
        String flag = StpUtil.getLoginIdAsLong() + "/" + "upload";
        String fileName = FileUtils.uploadSingleFile(file, filePath, flag);
        // 创建 FileMap 对象并保存到数据库
        FileMap fileMap = new FileMap()
                .setUrl("/profile/" + fileName)
                .setIsExist(true)
                .setIsLocal(true)
                .setIsWeb(false)
                .setLocalFilePath(fileName);
        // 将文件信息保存到数据库
        this.save(fileMap);
        return fileMap;
    }


    /**
     * 保存 BufferedImage 到本地服务器上
     *
     * @param image      BufferedImage 对象
     * @param formatName 图片格式（如 "png", "jpg"）
     * @return 上传结果
     */
    @Override
    public FileMap saveLocalFile(BufferedImage image, String formatName) {
        return saveLocalFile(image, IdUtil.getSnowflakeNextIdStr(), formatName);
    }


    /**
     * 保存 BufferedImage 到本地服务器上
     *
     * @param image      BufferedImage 对象
     * @param formatName 图片格式（如 "png", "jpg"）
     * @return 上传结果
     */
    @Override
    public FileMap saveLocalFile(BufferedImage image, String imageName, String formatName) {
        // 生成文件名
        String flag = StpUtil.getLoginIdAsLong() + "/" + "generate";
        String uniqueFileName = imageName + "." + formatName;
        String path =  flag + "/" + uniqueFileName;
        String fullPath = filePath + "/" + path;

        // 创建目标目录
        File directory = new File(filePath + "/" + flag);
        if (!directory.exists()) {
            directory.mkdirs();
        }

        // 保存 BufferedImage 为文件
        try {
            File outputFile = new File(fullPath);
            ImageIO.write(image, formatName, outputFile);
        } catch (IOException e) {
            throw new RuntimeException("保存图片失败", e);
        }

        // 创建 FileMap 对象并保存到数据库
        FileMap fileMap = new FileMap()
                .setUrl("/profile/" + path)
                .setIsExist(true)
                .setIsLocal(true)
                .setIsWeb(false)
                .setLocalFilePath(path);

        // 将文件信息保存到数据库
        this.save(fileMap);

        return fileMap;
    }

    /**
     * 获取本地路径的文件
     */
    @Override
    public String getLocalFilePath(FileMap fileMap) {
        if (!fileMap.getIsLocal()) {
            throw new BusinessException("这不是本地文件,请从云服务器里面获取！");
        }
        return filePath + fileMap.getLocalFilePath();
    }

    /**
     * 上传图片到云储存库当中
     */
    @Override
    public FileMap uploadOOS(MultipartFile file) {
        // 将 MultipartFile 转换为临时文件
        File tempFile = FileUtil.createTempFile(file.getOriginalFilename(), ".tmp", null, true);
        try {
            file.transferTo(tempFile);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return uploadOOS(tempFile);
    }

    @Override
    public FileMap uploadOOS(File file) {
        // 从 Redis 中获取管理员密钥
        String adminValue = redisCache.get(adminKey);
        if (StrUtil.isBlank(adminValue)) {
            redisCache.set(adminKey, adminPassword);
        }

        // 创建请求参数，将文件添加到请求中
        Map<String, Object> formData = new HashMap<>();
        formData.put("file", file);  // 传递 File 对象

        // 发起 POST 请求
        HttpResponse response = HttpRequest.post(adminUrl + "/admin-api/contact/file/upload")
                .form(formData)  // 发送表单参数
                .execute();

        JSONObject res = JSON.parseObject(response.body());
        // 检查响应状态并处理响应
        if (response.isOk() && res.getInteger("code") == 0) {
            // 创建 FileMap 对象并保存到数据库
            FileMap fileMap = new FileMap()
                    .setUrl(res.getString("data"))  // 这里假设 response.body() 返回的是文件的 URL
                    .setIsExist(true)
                    .setIsLocal(false)
                    .setIsWeb(true);

            // 保存文件信息到数据库
            this.save(fileMap);
            return fileMap;  // 返回成功的响应
        } else {
            throw new RuntimeException("上传失败: " + response.getStatus());
        }
    }

}
